@@ -14,7 +14,9 @@ from django.db import models
 
 from zeal.util import get_caller, get_stack
 
-from .errors import NPlusOneError, ZealError
+from .constants import ALL_APPS
+from .errors import NPlusOneError, ZealConfigError, ZealError
+from .signals import nplusone_detected
 
 
 class QuerySource(TypedDict):
@@ -32,6 +34,29 @@ class AllowListEntry(TypedDict):
     field: Optional[str]
 
 
+def _validate_allowlist(allowlist: list[AllowListEntry]):
+    for entry in allowlist:
+        fnmatch_chars = "*?[]"
+        # if this is an fnmatch, don't do anything
+        if any(char in entry["model"] for char in fnmatch_chars):
+            continue
+        if entry["model"] not in ALL_APPS:
+            raise ZealConfigError(
+                f"Model '{entry['model']}' not found in installed Django models"
+            )
+
+        if not entry["field"]:
+            continue
+
+        if any(char in entry["field"] for char in fnmatch_chars):
+            continue
+
+        if entry["field"] not in ALL_APPS[entry["model"]]:
+            raise ZealConfigError(
+                f"Field '{entry['field']}' not found on '{entry['model']}'"
+            )
+
+
 @dataclass
 class NPlusOneContext:
     enabled: bool = False
@@ -134,7 +159,7 @@ class NPlusOneListener(Listener):
         context.calls[key].append(get_stack())
         count = len(context.calls[key])
         if count >= self._threshold and instance_key not in context.ignored:
-            message = f"N+1 detected on {model.__name__}.{field}"
+            message = f"N+1 detected on {model._meta.app_label}.{model.__name__}.{field}"
             self._alert(model, field, message, context.calls[key])
         _nplusone_context.set(context)
 
@@ -158,6 +183,19 @@ class NPlusOneListener(Listener):
         else:
             return 2
 
+    def _alert(
+        self,
+        model: type[models.Model],
+        field: str,
+        message: str,
+        calls: list[list[inspect.FrameInfo]],
+    ):
+        super()._alert(model, field, message, calls)
+        nplusone_detected.send(
+            sender=self,
+            exception=self.error_class(message),
+        )
+
 
 n_plus_one_listener = NPlusOneListener()
 
@@ -166,6 +204,8 @@ def setup() -> Optional[Token]:
     # if we're already in an ignore-context, we don't want to override
     # it.
     context = _nplusone_context.get()
+    if hasattr(settings, "ZEAL_ALLOWLIST"):
+        _validate_allowlist(settings.ZEAL_ALLOWLIST)
     return _nplusone_context.set(
         NPlusOneContext(enabled=True, allowlist=context.allowlist)
     )
@@ -191,6 +231,8 @@ def zeal_context():
 def zeal_ignore(allowlist: Optional[list[AllowListEntry]] = None):
     if allowlist is None:
         allowlist = [{"model": "*", "field": "*"}]
+    else:
+        _validate_allowlist(allowlist)
 
     old_context = _nplusone_context.get()
     new_context = NPlusOneContext(
